﻿  /**
   * Merges an existing data provider with a new data provider. At the end of
   * the operation, the existing data provider has all the items that appear 
   * _both_ in itself _and_ in the new data provider. Use this method to update
   * a List component's contents without losing the current selection.
   *
   * @param   newDP       The new data provider.
   * @param   currentDP   The data provider to be updated.
   * @param   field       The field to use when comparing items in the
   *                      data providers (usually "data" or "label").
   */
  function consolidateProviders (newDP:Array, currentDP:Array, field:String) {
    // Prevent error due to insufficient data or circular reference.
    if (newDP == null || currentDP == null 
        || field == null || newDP == currentDP) {
      return;
    }

    // Get a snapshot of the current data provider.
    var oldDP:Array = currentDP.slice(0);

    // Variables for consolidation loops.
    var itemExists:Boolean;
    var itemIsGone:Boolean;
    var i:Number;
    var j:Number;
    var k:Number;

    // Add any new items that aren't already in the old list.
    // (Use prefix decrement for speed...)
    for (i = newDP.length; --i >= 0;) {
      itemExists = false;
      for (j = oldDP.length; --j >= 0;) {
        if (newDP[i][field] === oldDP[j][field]) {
          itemExists = true;
          break;
        }
      }
      if (!itemExists) {
        currentDP.addItem(newDP[i]);
      }
    }

    // Now remove any old items that aren't in the new list.
    // (Use prefix decrement for speed...)
    for (i = oldDP.length; --i >= 0;) {
      itemIsGone = true;
      for (j = newDP.length; --j >= 0;) {
        if (newDP[j][field] === oldDP[i][field]) {
          itemIsGone = false;
          break;
        }
      }
      if (itemIsGone) {
        // Find and remove the item.
        for (k = currentDP.length; --k >= 0;) {
          if (currentDP[k][field] === oldDP[i][field]) {
            currentDP.removeItemAt(k);
          }
        }
      }
    }
  }
